/*
 * Author: Radovan Cerveny
 */

#include "FactorOracleAutomaton.h"
#include <string/LinearString.h>

namespace stringology {

namespace exact {

automaton::Automaton FactorOracleAutomaton::construct ( const string::String & pattern ) {
    return dispatch ( pattern.getData ( ) );
}

automaton::DFA<> FactorOracleAutomaton::construct ( const string::LinearString < > & pattern ) {
    automaton::DFA<> oracle ( label::labelFrom ( 0 ) );

    oracle.addFinalState ( oracle.getInitialState ( ) );

    std::map < label::Label, label::Label > supplyFunction { { label::labelFrom ( 0 ), label::labelFrom ( -1 ) } };

    oracle.setInputAlphabet ( pattern.getAlphabet ( ) );

    for ( const alphabet::Symbol & symbol : pattern.getContent ( ) )
        oracleAddLetter ( oracle, symbol, supplyFunction );

    return oracle;
}

void FactorOracleAutomaton::oracleAddLetter ( automaton::DFA<> & oracle, const alphabet::Symbol & symbol, std::map < label::Label, label::Label > & supplyFunction ) {
    int m = ( int ) oracle.getStates ( ).size ( ) - 1;

    label::Label lastState = label::labelFrom ( m );
    label::Label newState = label::labelFrom ( m + 1 );

    oracle.addState ( newState );
    oracle.addFinalState ( newState );

    oracle.addTransition ( lastState, symbol, newState );
    label::Label kState = supplyFunction.find( lastState ) -> second;

    while ( kState != label::labelFrom ( -1 ) && oracle.getTransitions ( ).find ( { kState, symbol } ) == oracle.getTransitions ( ).end ( ) ) {
        oracle.addTransition ( kState, symbol, newState );
        kState = supplyFunction.find( kState ) -> second;
    }

    label::Label supplyState = label::labelFrom ( 0 );

    if ( kState != label::labelFrom ( -1 ) )
        supplyState = oracle.getTransitions ( ).find( { kState, symbol } ) -> second;

    supplyFunction.insert( { newState, supplyState } );
}

auto FactorOracleAutomatonLinearString = FactorOracleAutomaton::RegistratorWrapper < automaton::DFA<>, string::LinearString < > > ( FactorOracleAutomaton::construct );

} /* namespace exact */

} /* namespace stringology */
