/*
 * ranked_symbol.h
 *
 *  Created on: Apr 10, 2013
 *      Author: Stepan Plachy
 */

#ifndef RAW_RANKED_SYMBOL_H_
#define RAW_RANKED_SYMBOL_H_

#include <sax/FromXMLParserHelper.h>
#include "SymbolFeatures.h"
#include "Symbol.h"

namespace std {

/**
 * Represents symbol in an std.
 */
template < class SymbolType = alphabet::Symbol, class RankType = primitive::Unsigned >
class ranked_symbol {
protected:
	SymbolType symbol;
	RankType rank;

public:
	explicit ranked_symbol ( int number, unsigned rank );
	explicit ranked_symbol ( char character, unsigned rank );
	explicit ranked_symbol ( std::string label, unsigned rank );

	/**
	 * Creates new symbol with given name and rank.
	 * @param symbol name of the symbol
	 * @param rank of the symbol
	 */
	explicit ranked_symbol ( SymbolType symbol, RankType rank );
	explicit ranked_symbol ( SymbolType symbol, unsigned rank );

	/**
	 * @return name of the symbol
	 */
	const SymbolType & getSymbol ( ) const;

	/**
	 * @return rank of the symbol
	 */
	const RankType & getRank ( ) const;

	int compare ( const ranked_symbol & other ) const;

	explicit operator std::string ( ) const;

	bool operator < ( const ranked_symbol & other ) const {
		return compare ( other ) < 0;
	}

	bool operator > ( const ranked_symbol & other ) const {
		return compare ( other ) > 0;
	}

	bool operator <= ( const ranked_symbol & other ) const {
		return compare ( other ) <= 0;
	}

	bool operator >= ( const ranked_symbol & other ) const {
		return compare ( other ) >= 0;
	}

	bool operator == ( const ranked_symbol & other ) const {
		return compare ( other ) == 0;
	}

	bool operator != ( const ranked_symbol & other ) const {
		return compare ( other ) != 0;
	}
};

template < class SymbolType, class RankType >
ranked_symbol < SymbolType, RankType >::ranked_symbol(int number, unsigned rank) : symbol(alphabet::symbolFrom(number)), rank(RankType(rank)) {

}

template < class SymbolType, class RankType >
ranked_symbol < SymbolType, RankType >::ranked_symbol(char character, unsigned rank) : symbol(alphabet::symbolFrom(character)), rank(RankType(rank)) {

}

template < class SymbolType, class RankType >
ranked_symbol < SymbolType, RankType >::ranked_symbol(std::string symbol, unsigned rank) : symbol(alphabet::symbolFrom(std::move(symbol))), rank(RankType(rank)) {

}

template < class SymbolType, class RankType >
ranked_symbol < SymbolType, RankType >::ranked_symbol(SymbolType symbol, RankType rank) : symbol(std::move(symbol)), rank(std::move(rank)) {

}

template < class SymbolType, class RankType >
ranked_symbol < SymbolType, RankType >::ranked_symbol(SymbolType symbol, unsigned rank) : symbol(std::move(symbol)), rank(RankType(rank)) {

}

template < class SymbolType, class RankType >
const SymbolType& ranked_symbol < SymbolType, RankType >::getSymbol() const {
	return symbol;
}

template < class SymbolType, class RankType >
const RankType& ranked_symbol < SymbolType, RankType >::getRank() const {
	return rank;
}

template < class SymbolType, class RankType >
int ranked_symbol < SymbolType, RankType >::compare(const ranked_symbol& other) const {
	auto first = std::tie ( symbol, rank );
	auto second = std::tie ( other.symbol, other.rank );

	std::compare<decltype(first)> comp;
	return comp(first, second);
}

template < class SymbolType, class RankType >
ranked_symbol < SymbolType, RankType >::operator std::string () const {
	return (std::string) symbol + "_" + (std::string) rank;
}

} /* namespace std */

namespace std {

template < class SymbolType, class RankType >
std::ostream & operator << ( std::ostream & out, const std::ranked_symbol < SymbolType, RankType > & symbol ) {
	out << "(ranked_symbol " << symbol.getSymbol ( ) << " #" << symbol.getRank ( ) << ")";
	return out;
}

template < class T, class R >
struct compare < std::ranked_symbol < T, R > > {
	int operator ()( const std::ranked_symbol < T, R > & first, const std::ranked_symbol < T, R > & second ) const {
		return first.compare ( second );
	}

};

} /* namespace std */

#endif /* RAW_RANKED_SYMBOL_H_ */
