/*
 * FinalStateLabel.h
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travnicek
 */

#ifndef FINAL_STATE_LABEL_H_
#define FINAL_STATE_LABEL_H_

#include "LabelBase.h"
#include "Label.h"

namespace label {

/**
 * Represents symbol in an alphabet.
 */
class FinalStateLabel : public LabelBase {
public:
	/**
	 * Creates new symbol with given name.
	 * @param symbol name of the symbol
	 */
	explicit FinalStateLabel ( );

	virtual LabelBase * clone ( ) const;

	virtual LabelBase * plunder ( ) &&;

	virtual int compare ( const ObjectBase & other ) const {
		if ( std::type_index ( typeid ( * this ) ) == std::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return std::type_index ( typeid ( * this ) ) - std::type_index ( typeid ( other ) );
	}

	virtual int compare ( const FinalStateLabel & other ) const;

	virtual void operator >>( std::ostream & ) const;

	virtual explicit operator std::string ( ) const;

	static FinalStateLabel FINAL_STATE;
	static Label FINAL_STATE_LABEL;

	static const std::string & getXmlTagName() {
		static std::string xmlTagName = "FinalStateLabel";

		return xmlTagName;
	}

	static FinalStateLabel parse ( std::deque < sax::Token >::iterator & input );

	void compose ( std::deque < sax::Token > & out ) const;

	virtual LabelBase * next ( ) const;
	virtual LabelBase * inc ( ) &&;
};

} /* namespace label */

#endif /* FINAL_STATE_LABEL_H_ */
