/*
 * Label.cpp
 *
 *  Created on: Apr 16, 2013
 *      Author: Martin Zak
 */

#include "Label.h"
#include "PrimitiveLabel.h"
#include "LabelPairLabel.h"
#include <primitive/Primitive.h>
#include <core/xmlApi.hpp>
#include <exception/CommonException.h>
#include <climits>

namespace label {

Label Label::next ( ) const {
	return Label ( this->getData ( ).next ( ) );
}

void Label::inc ( ) {
	LabelBase * res = std::move ( this->getData ( ) ).inc ( );

	if ( res == NULL ) return;

	this->setData ( res );
}

label::Label createUniqueLabel ( label::Label nextState, const std::set < label::Label > & other ) {
	int i = 0;

	do {
		if ( other.count ( nextState ) == 0 )
			return nextState;

		nextState.inc ( );
	} while ( i++ < INT_MAX );

	throw exception::CommonException ( "Could not create unique label." );
}

label::Label labelFrom ( int number ) {
	return label::Label {
			   label::PrimitiveLabel {
				   primitive::primitiveFrom ( number )
			   }
	};
}

label::Label labelFrom ( int number1, int number2 ) {
	return label::Label {
			label::LabelPairLabel {
				std::make_pair ( label::Label { label::PrimitiveLabel { primitive::primitiveFrom ( number1 ) } },
						 label::Label { label::PrimitiveLabel { primitive::primitiveFrom ( number2 ) } }
				)
			}
	};
}

label::Label labelFrom ( label::Label label1, label::Label label2 ) {
	return label::Label {
			label::LabelPairLabel {
				std::make_pair ( std::move ( label1 ),
						 std::move ( label2 )
				)
			}
	};
}

label::Label labelFrom ( char character ) {
	return label::Label {
			   label::PrimitiveLabel {
				   primitive::primitiveFrom ( character )
			   }
	};
}

label::Label labelFrom ( std::string string ) {
	return label::Label {
			   label::PrimitiveLabel {
				   primitive::primitiveFrom ( std::move ( string ) )
			   }
	};
}

label::Label labelFrom ( const char * string ) {
	return labelFrom ( ( std::string ) string );
}

} /* namespace label */

namespace alib {

auto LabelDeleter = xmlApi < label::Label >::InputContextDeleter ( );
}
