#include "RTEAlphabet.h"

#include <algorithm>
#include <exception/CommonException.h>
#include <primitive/Unsigned.h>

namespace rte {

bool RTEAlphabet::addSymbolToAlphabet ( std::ranked_symbol < > symbol ) {
	return alphabetF.insert ( std::move ( symbol ) ).second;
}

bool RTEAlphabet::addConstantSymbolToAlphabet ( std::ranked_symbol < > symbol ) {
	if ( symbol.getRank ( ) != primitive::Unsigned ( 0 ) )
		throw exception::CommonException ( "RTE's constant alphabet can contain only symbols with arity zero." );

	return alphabetK.insert ( std::move ( symbol ) ).second;
}

void RTEAlphabet::addSymbolsToAlphabet ( const std::set < std::ranked_symbol < > > & symbols ) {
	for ( const std::ranked_symbol < > & addedSymbol : symbols )
		addSymbolToAlphabet ( addedSymbol );
}

void RTEAlphabet::addConstantSymbolsToAlphabet ( const std::set < std::ranked_symbol < > > & symbols ) {
	for ( const std::ranked_symbol < > & addedSymbol : symbols )
		addConstantSymbolToAlphabet ( addedSymbol );
}

void RTEAlphabet::setAlphabet ( std::set < std::ranked_symbol < > > newSymbols ) {
	std::set < std::ranked_symbol < > > removed;
	std::set_difference ( alphabetF.begin ( ), alphabetF.end ( ), newSymbols.begin ( ), newSymbols.end ( ), std::inserter ( removed, removed.end ( ) ) );

	for ( const std::ranked_symbol < > & removedSymbol : removed )
		removeSymbolFromAlphabet ( removedSymbol );

	alphabetF = std::move ( newSymbols );
}

void RTEAlphabet::setConstantAlphabet ( std::set < std::ranked_symbol < > > newSymbols ) {
	std::set < std::ranked_symbol < > > removed;
	std::set_difference ( alphabetK.begin ( ), alphabetK.end ( ), newSymbols.begin ( ), newSymbols.end ( ), std::inserter ( removed, removed.end ( ) ) );

	for ( const std::ranked_symbol < > & removedSymbol : removed )
		removeConstantSymbolFromAlphabet ( removedSymbol );

	alphabetK = std::move ( newSymbols );
}

const std::set < std::ranked_symbol < > > & RTEAlphabet::getAlphabet ( ) const {
	return alphabetF;
}

const std::set < std::ranked_symbol < > > & RTEAlphabet::getSubstitutionAlphabet ( ) const {
	return alphabetK;
}

} /* namespace rte */
