#ifndef RTE_ALPHABET_H_
#define RTE_ALPHABET_H_

#include <set>
#include <alphabet/ranked_symbol.hpp>

namespace rte {

/**
 * Abstract base class for all strings. Contains common elements of strings.
 */
class RTEAlphabet {
protected:
	std::set < std::ranked_symbol < > > alphabetF, alphabetK;

public:
	/**
	 * Adds input symbol to input alphabet.
	 * @param symbol Symbol to add
	 */
	bool addSymbolToAlphabet ( std::ranked_symbol < > symbol );
	bool addConstantSymbolToAlphabet ( std::ranked_symbol < > symbol );

	/**
	 * Adds input symbols to input alphabet.
	 * @param symbols Symbol to add
	 */
	void addSymbolsToAlphabet ( const std::set < std::ranked_symbol < > > & symbols );
	void addConstantSymbolsToAlphabet ( const std::set < std::ranked_symbol < > > & symbols );

	/**
	 * Sets input symbols of the tree.
	 * @param symbols Symbols to set
	 */
	void setAlphabet ( std::set < std::ranked_symbol < > > symbols );
	void setConstantAlphabet ( std::set < std::ranked_symbol < > > symbols );

	/**
	 * Removes input symbol from the input alphabet.
	 * @param symbol Symbol to remove
	 * @throws AutomatonException when symbol is not present in input alphabet
	 * or when symbol is part of the transition
	 */
	virtual bool removeSymbolFromAlphabet ( const std::ranked_symbol < > & symbol ) = 0;
	virtual bool removeConstantSymbolFromAlphabet ( const std::ranked_symbol < > & symbol ) = 0;

	/**
	 * @return the input alphabet
	 */
	const std::set < std::ranked_symbol < > > & getAlphabet ( ) const;
	const std::set < std::ranked_symbol < > > & getSubstitutionAlphabet ( ) const;
};

} /* namespace rte */

#endif /* RTE_ALPHABET_H_ */
