#include "RTEToXMLComposer.h"

#include "../RTE.h"
#include "../RTEClasses.h"

#include "../formal/FormalRTEElements.h"
#include <core/xmlApi.hpp>
#include <alphabet/RankedSymbol.h>

namespace rte {

void RTEToXMLComposer::Visit ( void * userData, const FormalRTEAlternation & alternation ) const {
	std::deque < sax::Token > & out = * ( ( std::deque < sax::Token > * )userData );

	out.emplace_back ( "alternation", sax::Token::TokenType::START_ELEMENT );
	alternation.getLeftElement ( ).Accept ( userData, * this );
	alternation.getRightElement ( ).Accept ( userData, * this );
	out.emplace_back ( "alternation", sax::Token::TokenType::END_ELEMENT );
}

void RTEToXMLComposer::Visit ( void * userData, const FormalRTESubstitution & substitution ) const {
	std::deque < sax::Token > & out = * ( ( std::deque < sax::Token > * )userData );

	out.emplace_back ( "substitution", sax::Token::TokenType::START_ELEMENT );

	alib::xmlApi < std::ranked_symbol < > >::compose ( out, substitution.getSubstitutionSymbol ( ).getSymbol ( ) );
	substitution.getLeftElement ( ).Accept ( userData, * this );
	substitution.getRightElement ( ).Accept ( userData, * this );

	out.emplace_back ( "substitution", sax::Token::TokenType::END_ELEMENT );
}

void RTEToXMLComposer::Visit ( void * userData, const FormalRTEIteration & iteration ) const {
	std::deque < sax::Token > & out = * ( ( std::deque < sax::Token > * )userData );

	out.emplace_back ( "iteration", sax::Token::TokenType::START_ELEMENT );

	alib::xmlApi < std::ranked_symbol < > >::compose ( out, iteration.getSubstitutionSymbol ( ).getSymbol ( ) );
	iteration.getElement ( ).Accept ( userData, * this );

	out.emplace_back ( "iteration", sax::Token::TokenType::END_ELEMENT );
}

void RTEToXMLComposer::Visit ( void * userData, const FormalRTESymbolAlphabet & symbol ) const {
	std::deque < sax::Token > & out = * ( ( std::deque < sax::Token > * )userData );

	out.emplace_back ( sax::Token ( "symbol", sax::Token::TokenType::START_ELEMENT ) );
	alib::xmlApi < std::ranked_symbol < > >::compose ( out, symbol.getSymbol ( ) );

	for ( const std::smart_ptr < const FormalRTESymbol > & element : symbol.getElements ( ) )
		element->Accept ( userData, * this );

	out.emplace_back ( sax::Token ( "symbol", sax::Token::TokenType::END_ELEMENT ) );
}

void RTEToXMLComposer::Visit ( void * userData, const FormalRTESymbolSubst & symbol ) const {
	std::deque < sax::Token > & out = * ( ( std::deque < sax::Token > * )userData );

	out.emplace_back ( sax::Token ( "substSymbol", sax::Token::TokenType::START_ELEMENT ) );
	alib::xmlApi < std::ranked_symbol < > >::compose ( out, symbol.getSymbol ( ) );

	out.emplace_back ( sax::Token ( "substSymbol", sax::Token::TokenType::END_ELEMENT ) );
}

void RTEToXMLComposer::Visit ( void * userData, const FormalRTEEmpty & ) const {
	std::deque < sax::Token > & out = * ( ( std::deque < sax::Token > * )userData );

	out.emplace_back ( "empty", sax::Token::TokenType::START_ELEMENT );
	out.emplace_back ( "empty", sax::Token::TokenType::END_ELEMENT );
}

void RTEToXMLComposer::composeAlphabet ( std::deque < sax::Token > & out, const std::set < std::ranked_symbol < > > & alphabetF, const std::set < std::ranked_symbol < > > & alphabetK ) {
	out.emplace_back ( "alphabet", sax::Token::TokenType::START_ELEMENT );

	for ( const auto & symbol : alphabetF )
		alib::xmlApi < std::ranked_symbol < > >::compose ( out, symbol );

	out.emplace_back ( "alphabet", sax::Token::TokenType::END_ELEMENT );

	out.emplace_back ( "substSymbolAlphabet", sax::Token::TokenType::START_ELEMENT );

	for ( const auto & symbol : alphabetK )
		alib::xmlApi < std::ranked_symbol < > >::compose ( out, symbol );

	out.emplace_back ( "substSymbolAlphabet", sax::Token::TokenType::END_ELEMENT );
}

RTEToXMLComposer RTEToXMLComposer::RTE_TO_XML_COMPOSER;

} /* namespace rte */
