#include "FormalRTE.h"
#include "FormalRTEEmpty.h"
#include "FormalRTESymbol.h"
#include <exception/CommonException.h>

#include <algorithm>
#include <iostream>
#include <sstream>

#include "../RTE.h"
#include "../common/RTEFromXMLParser.h"
#include "../common/RTEToXMLComposer.h"
#include <core/xmlApi.hpp>
#include <core/castApi.hpp>
#include <object/Object.h>
#include <sax/FromXMLParserHelper.h>

namespace rte {

FormalRTE::FormalRTE ( std::set < std::ranked_symbol < > > alphabetF, std::set < std::ranked_symbol < > > alphabetK, FormalRTEElement && rte ) : std::Components < FormalRTE, std::ranked_symbol < >, std::tuple < GeneralAlphabet, ConstantAlphabet >, std::tuple < > > ( std::make_tuple ( std::move ( alphabetF ), std::move ( alphabetK ) ), std::tuple < > ( ) ), rte ( NULL ) {
	setRTE ( std::move ( rte ) );
}

FormalRTE::FormalRTE ( std::pair < std::set < std::ranked_symbol < > >, std::set < std::ranked_symbol < > > > alphabets, FormalRTEElement && rte ) : FormalRTE ( std::move ( alphabets.first ), std::move ( alphabets.second ), std::move ( rte ) ) {
}

FormalRTE::FormalRTE ( std::pair < std::set < std::ranked_symbol < > >, std::set < std::ranked_symbol < > > > alphabets, const FormalRTEElement & rte ) : FormalRTE ( std::move ( alphabets.first ), std::move ( alphabets.second ), std::move_copy ( rte ) ) {
}

FormalRTE::FormalRTE ( ) : FormalRTE ( std::set < std::ranked_symbol < > > ( ), std::set < std::ranked_symbol < > > ( ), FormalRTEEmpty ( ) ) {
}

FormalRTE::FormalRTE ( std::set < std::ranked_symbol < > > alphabetF, std::set < std::ranked_symbol < > > alphabetK, const FormalRTEElement & rte ) : FormalRTE ( std::move ( alphabetF ), std::move ( alphabetK ), std::move_copy ( rte ) ) {
}

FormalRTE::FormalRTE ( const FormalRTEElement & rte ) : FormalRTE ( rte.computeMinimalAlphabets ( ), std::move_copy ( rte ) ) {
}

FormalRTE::FormalRTE ( FormalRTEElement && rte ) : FormalRTE ( rte.computeMinimalAlphabets ( ), rte ) {
}

RTEBase * FormalRTE::clone ( ) const {
	return new FormalRTE ( * this );
}

RTEBase * FormalRTE::plunder ( ) && {
	return new FormalRTE ( std::move ( * this ) );
}

const FormalRTEElement & FormalRTE::getRTE ( ) const {
	return * rte;
}

FormalRTEElement & FormalRTE::getRTE ( ) {
	return * rte;
}

void FormalRTE::setRTE ( const FormalRTEElement & rte ) {
	setRTE ( std::move_copy ( rte ) );
}

void FormalRTE::setRTE ( FormalRTEElement && param ) {
	this->rte = std::smart_ptr < FormalRTEElement > ( std::move ( param ).plunder ( ) );

	if ( !this->rte->checkAlphabet ( getAlphabet ( ), getSubstitutionAlphabet ( ) ) )
		throw exception::CommonException ( "Symbols not matching alphabets." );
}

void FormalRTE::operator >>( std::ostream & out ) const {
	out << "(FormalRTE " << * ( this->rte ) << ")";
}

int FormalRTE::compare ( const FormalRTE & other ) const {
	auto first = std::tie ( rte, getAlphabet ( ), getSubstitutionAlphabet ( ) );
	auto second = std::tie ( rte, other.getAlphabet ( ), getSubstitutionAlphabet ( ) );

	std::compare < decltype ( first ) > comp;

	return comp ( first, second );
}

FormalRTE::operator std::string ( ) const {
	std::stringstream ss;
	ss << * this;
	return ss.str ( );
}

FormalRTE FormalRTE::parse ( std::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, FormalRTE::getXmlTagName ( ) );

	std::pair < std::set < std::ranked_symbol < > >, std::set < std::ranked_symbol < > > > alphabets = RTEFromXMLParser::parseAlphabet ( input );
	std::rvalue_ref < FormalRTEElement > element = alib::xmlApi < rte::FormalRTEElement >::parse ( input );
	FormalRTE rte ( std::move ( alphabets.first ), std::move ( alphabets.second ), std::move ( element ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, FormalRTE::getXmlTagName ( ) );
	return rte;
}

void FormalRTE::compose ( std::deque < sax::Token > & out ) const {
	out.emplace_back ( FormalRTE::getXmlTagName ( ), sax::Token::TokenType::START_ELEMENT );
	RTEToXMLComposer::composeAlphabet ( out, this->getAlphabet ( ), this->getSubstitutionAlphabet ( ) );
	alib::xmlApi < FormalRTEElement >::compose ( out, this->getRTE ( ) );
	out.emplace_back ( FormalRTE::getXmlTagName ( ), sax::Token::TokenType::END_ELEMENT );
}

} /* namespace rte */

namespace alib {

auto formalRTEParserRegister = xmlApi < rte::RTE >::ParserRegister < rte::FormalRTE > ( );
auto formalRTEParserRegister2 = xmlApi < alib::Object >::ParserRegister < rte::FormalRTE > ( );

auto FormalRTECastBinder = castApi::CastPoolStringBinder < rte::FormalRTE > ( rte::FormalRTE::getXmlTagName ( ) );

} /* namespace alib */
