#include "FormalRTEAlternation.h"
#include <exception/CommonException.h>

#include <sstream>

namespace rte {

FormalRTEAlternation::FormalRTEAlternation ( FormalRTEElement && left, FormalRTEElement && right ) : BinaryNode < std::smart_ptr < FormalRTEElement >, std::smart_ptr < FormalRTEElement >, FormalRTEAlternation > ( std::smart_ptr < FormalRTEElement > ( std::move ( left ).plunder ( ) ), std::smart_ptr < FormalRTEElement > ( std::move ( right ).plunder ( ) ) ) {
}

FormalRTEAlternation::FormalRTEAlternation ( const FormalRTEElement & left, const FormalRTEElement & right ) : FormalRTEAlternation ( std::move_copy ( left ), std::move_copy ( right ) ) {
}

const FormalRTEElement & FormalRTEAlternation::getLeftElement ( ) const {
	return * getLeft ( );
}

const FormalRTEElement & FormalRTEAlternation::getRightElement ( ) const {
	return * getRight ( );
}

FormalRTEElement & FormalRTEAlternation::getLeftElement ( ) {
	return * getLeft ( );
}

FormalRTEElement & FormalRTEAlternation::getRightElement ( ) {
	return * getRight ( );
}

void FormalRTEAlternation::setLeftElement ( const FormalRTEElement & element ) {
	setLeftElement ( std::move_copy ( element ) );
}

void FormalRTEAlternation::setLeftElement ( FormalRTEElement && element ) {
	setLeft ( std::smart_ptr < FormalRTEElement > ( std::move ( element ).plunder ( ) ) );
}

void FormalRTEAlternation::setRightElement ( const FormalRTEElement & element ) {
	setRightElement ( std::move_copy ( element ) );
}

void FormalRTEAlternation::setRightElement ( FormalRTEElement && element ) {
	setRight ( std::smart_ptr < FormalRTEElement > ( std::move ( element ).plunder ( ) ) );
}

FormalRTEAlternation * FormalRTEAlternation::clone ( ) const {
	return new FormalRTEAlternation ( * this );
}

FormalRTEAlternation * FormalRTEAlternation::plunder ( ) && {
	return new FormalRTEAlternation ( std::move ( * this ) );
}

int FormalRTEAlternation::compare ( const FormalRTEAlternation & other ) const {
	int res = getLeftElement ( ).compare ( other.getLeftElement ( ) );

	if ( res == 0 ) res = getRightElement ( ).compare ( other.getRightElement ( ) );

	return res;
}

void FormalRTEAlternation::operator >>( std::ostream & out ) const {
	out << "(FormalRTEAlternation";
	out << " " << getLeftElement ( );
	out << " " << getRightElement ( );
	out << ")";
}

bool FormalRTEAlternation::testSymbol ( const std::ranked_symbol < > & symbol ) const {
	if ( getLeftElement ( ).testSymbol ( symbol ) ) return true;

	if ( getRightElement ( ).testSymbol ( symbol ) ) return true;

	return false;
}

void FormalRTEAlternation::computeMinimalAlphabet ( std::set < std::ranked_symbol < > > & alphabetF, std::set < std::ranked_symbol < > > & alphabetK ) const {
	getLeftElement ( ).computeMinimalAlphabet ( alphabetF, alphabetK );
	getRightElement ( ).computeMinimalAlphabet ( alphabetF, alphabetK );
}

bool FormalRTEAlternation::checkAlphabet ( const std::set < std::ranked_symbol < > > & alphabetF, const std::set < std::ranked_symbol < > > & alphabetK ) const {
	return getLeftElement ( ).checkAlphabet ( alphabetF, alphabetK ) && getRightElement ( ).checkAlphabet ( alphabetF, alphabetK );
}

FormalRTEAlternation::operator std::string ( ) const {
	std::stringstream ss;
	ss << * this;
	return ss.str ( );
}

} /* namespace rte */
