#ifndef FORMAL_RTE_ALTERNATION_H_
#define FORMAL_RTE_ALTERNATION_H_

#include "FormalRTEElement.h"

namespace rte {

/**
 * Represents alternation operator in the regular expression. Contains list of FormalRTEElement
 * as operands of the operator.
 */
class FormalRTEAlternation : public FormalRTEElement, public std::BinaryNode < std::smart_ptr < FormalRTEElement >, std::smart_ptr < FormalRTEElement >, FormalRTEAlternation > {
public:
	void Accept ( void * userData, const FormalRTEElementVisitor & visitor ) const {
		visitor.Visit ( userData, * this );
	}

	explicit FormalRTEAlternation ( FormalRTEElement && left, FormalRTEElement && right );
	explicit FormalRTEAlternation ( const FormalRTEElement & left, const FormalRTEElement & right );

	/**
	 * @copydoc FormalRTEElement::clone() const
	 */
	virtual FormalRTEAlternation * clone ( ) const;

	/**
	 * @copydoc FormalRTEElement::plunder() const
	 */
	virtual FormalRTEAlternation * plunder ( ) &&;

	/**
	 * @copydoc FormalRTEElement::testSymbol() const
	 */
	virtual bool testSymbol ( const std::ranked_symbol < > & symbol ) const;

	/**
	 * @copydoc FormalRTEElement::computeMinimalAlphabet()
	 */
	virtual void computeMinimalAlphabet ( std::set < std::ranked_symbol < > > & alphabetF, std::set < std::ranked_symbol < > > & alphabetK ) const;

	/**
	 * @copydoc FormalRTEElement::checkAlphabet()
	 */
	virtual bool checkAlphabet ( const std::set < std::ranked_symbol < > > & alphabetF, const std::set < std::ranked_symbol < > > & alphabetK ) const;

	/**
	 * @return elements
	 */
	const FormalRTEElement & getLeftElement ( ) const;
	const FormalRTEElement & getRightElement ( ) const;

	/**
	 * @return elements
	 */
	FormalRTEElement & getLeftElement ( );
	FormalRTEElement & getRightElement ( );

	/**
	 * @param element to append
	 */
	void setLeftElement ( const FormalRTEElement & element );
	void setLeftElement ( FormalRTEElement && element );

	/**
	 * @param element to append
	 */
	void setRightElement ( const FormalRTEElement & element );
	void setRightElement ( FormalRTEElement && element );

	virtual int compare ( const FormalRTEElement & other ) const {
		if ( std::type_index ( typeid ( * this ) ) == std::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return std::type_index ( typeid ( * this ) ) - std::type_index ( typeid ( other ) );
	}

	virtual int compare ( const FormalRTEAlternation & ) const;

	/**
	 * @copydoc FormalRTEElement::operator>>() const
	 */
	virtual void operator >>( std::ostream & out ) const;

	virtual explicit operator std::string ( ) const;
};

} /* namespace rte */

#endif /* FORMAL_RTE_ALTERNATION_H_ */
