#include "FormalRTEIteration.h"
#include <exception/CommonException.h>

#include <sstream>

namespace rte {

FormalRTEIteration::FormalRTEIteration ( FormalRTEElement && element, FormalRTESymbolSubst substitutionSymbol ) : std::UnaryNode < std::smart_ptr < FormalRTEElement >, std::smart_ptr < const FormalRTEElement >, FormalRTEIteration > ( std::smart_ptr < FormalRTEElement > ( std::move ( element ).plunder ( ) ) ), substitutionSymbol ( std::smart_ptr < FormalRTESymbolSubst > ( std::move ( substitutionSymbol ).plunder ( ) ) ) {
}

FormalRTEIteration::FormalRTEIteration ( const FormalRTEElement & element, FormalRTESymbolSubst substitutionSymbol ) : FormalRTEIteration ( std::move_copy ( element ), std::move ( substitutionSymbol ) ) {
}

const FormalRTEElement & FormalRTEIteration::getElement ( ) const {
	return * getChild ( );
}

FormalRTEElement & FormalRTEIteration::getElement ( ) {
	return * getChild ( );
}

const FormalRTESymbolSubst & FormalRTEIteration::getSubstitutionSymbol ( ) const {
	return * substitutionSymbol;
}

FormalRTESymbolSubst & FormalRTEIteration::getSubstitutionSymbol ( ) {
	return * substitutionSymbol;
}

void FormalRTEIteration::setElement ( const FormalRTEElement & element ) {
	setElement ( std::move_copy ( element ) );
}

void FormalRTEIteration::setElement ( FormalRTEElement && elementParam ) {
	setChild ( std::smart_ptr < FormalRTEElement > ( std::move ( elementParam ).plunder ( ) ) );
}

void FormalRTEIteration::setSubstitutionSymbol ( FormalRTESymbolSubst symbol ) {
	substitutionSymbol = std::smart_ptr < FormalRTESymbolSubst > ( std::move ( symbol ).plunder ( ) );
}

FormalRTEIteration * FormalRTEIteration::clone ( ) const {
	return new FormalRTEIteration ( * this );
}

FormalRTEIteration * FormalRTEIteration::plunder ( ) && {
	return new FormalRTEIteration ( std::move ( * this ) );
}

int FormalRTEIteration::compare ( const FormalRTEIteration & other ) const {
	auto first = std::tie ( getElement ( ), getSubstitutionSymbol ( ) );
	auto second = std::tie ( other.getElement ( ), other.getSubstitutionSymbol ( ) );

	std::compare < decltype ( first ) > comp;

	return comp ( first, second );
}

void FormalRTEIteration::operator >>( std::ostream & out ) const {
	out << "(RTEFormalRTEIteration " << * substitutionSymbol << " " << getElement ( ) << ")";
}

bool FormalRTEIteration::testSymbol ( const std::ranked_symbol < > & symbol ) const {
	return getChild ( )->testSymbol ( symbol );
}

void FormalRTEIteration::computeMinimalAlphabet ( std::set < std::ranked_symbol < > > & alphabetF, std::set < std::ranked_symbol < > > & alphabetK ) const {
	alphabetK.insert ( substitutionSymbol->getSymbol ( ) );
	getChild ( )->computeMinimalAlphabet ( alphabetF, alphabetK );
}

bool FormalRTEIteration::checkAlphabet ( const std::set < std::ranked_symbol < > > & alphabetF, const std::set < std::ranked_symbol < > > & alphabetK ) const {
	return alphabetK.count ( getSubstitutionSymbol ( ).getSymbol ( ) ) > 0 && getElement ( ).checkAlphabet ( alphabetF, alphabetK );
}

FormalRTEIteration::operator std::string ( ) const {
	std::stringstream ss;
	ss << * this;
	return ss.str ( );
}

} /* namespace rte */
