#ifndef FORMAL_RTE_ITERATION_H_
#define FORMAL_RTE_ITERATION_H_

#include "FormalRTE.h"
#include "FormalRTEElement.h"
#include "FormalRTESymbolSubst.h"

namespace rte {

namespace simplify {

class RTEOptimize;

} /* namespace simplify */

/**
 * Represents iteration operator in the regular expression. Contains one FormalRTEElement
 * as operand.
 */
class FormalRTEIteration : public FormalRTEElement, public std::UnaryNode < std::smart_ptr < FormalRTEElement >, std::smart_ptr < const FormalRTEElement >, FormalRTEIteration > {
protected:
	std::smart_ptr < FormalRTESymbolSubst > substitutionSymbol;

public:
	void Accept ( void * userData, const FormalRTEElementVisitor & visitor ) const {
		visitor.Visit ( userData, * this );
	}

	explicit FormalRTEIteration ( FormalRTEElement &&, FormalRTESymbolSubst );
	explicit FormalRTEIteration ( const FormalRTEElement &, FormalRTESymbolSubst );

	/**
	 * @copydoc FormalRTEElement::clone() const
	 */
	virtual FormalRTEIteration * clone ( ) const;

	/**
	 * @copydoc FormalRTEElement::plunder() const
	 */
	virtual FormalRTEIteration * plunder ( ) &&;

	/**
	 * @copydoc FormalRTEElement::testSymbol() const
	 */
	virtual bool testSymbol ( const std::ranked_symbol < > & symbol ) const;

	/**
	 * @copydoc FormalRTEElement::computeMinimalAlphabet()
	 */
	virtual void computeMinimalAlphabet ( std::set < std::ranked_symbol < > > & alphabetF, std::set < std::ranked_symbol < > > & alphabetK ) const;

	/**
	 * @copydoc FormalRTEElement::checkAlphabet()
	 */
	virtual bool checkAlphabet ( const std::set < std::ranked_symbol < > > & alphabetF, const std::set < std::ranked_symbol < > > & alphabetK ) const;

	/**
	 * @return element
	 */
	const FormalRTEElement & getElement ( ) const;
	const FormalRTESymbolSubst & getSubstitutionSymbol ( ) const;

	/**
	 * @return element
	 */
	FormalRTEElement & getElement ( );
	FormalRTESymbolSubst & getSubstitutionSymbol ( );

	/**
	 * @param element to iterate
	 */
	void setElement ( const FormalRTEElement & element );
	void setElement ( FormalRTEElement && element );

	void setSubstitutionSymbol ( FormalRTESymbolSubst element );

	virtual int compare ( const FormalRTEElement & other ) const {
		if ( std::type_index ( typeid ( * this ) ) == std::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return std::type_index ( typeid ( * this ) ) - std::type_index ( typeid ( other ) );
	}

	virtual int compare ( const FormalRTEIteration & ) const;

	/**
	 * @copydoc FormalRTEElement::operator>>() const
	 */
	virtual void operator >>( std::ostream & out ) const;

	friend class simplify::RTEOptimize;

	virtual explicit operator std::string ( ) const;
};

} /* namespace rte */

#endif /* FORMAL_RTE_ITERATION_H_ */
