#ifndef FORMAL_RTE_SUBSTITUTION_H_
#define FORMAL_RTE_SUBSTITUTION_H_

#include "FormalRTE.h"
#include "FormalRTEElement.h"
#include "FormalRTESymbolSubst.h"

namespace rte {

/**
 * Represents concatenation operator in the regular expression. Contains list of FormalRTEElement
 * as operands of the operator.
 */
class FormalRTESubstitution : public FormalRTEElement, public std::BinaryNode < std::smart_ptr < FormalRTEElement >, std::smart_ptr < FormalRTEElement >, FormalRTESubstitution > {
protected:
	std::smart_ptr < FormalRTESymbolSubst > substitutionSymbol; // substite this in left by right

public:
	void Accept ( void * userData, const FormalRTEElementVisitor & visitor ) const {
		visitor.Visit ( userData, * this );
	}

	explicit FormalRTESubstitution ( FormalRTEElement && left, FormalRTEElement && right, FormalRTESymbolSubst substitutionSymbol );
	explicit FormalRTESubstitution ( const FormalRTEElement & left, const FormalRTEElement & right, FormalRTESymbolSubst substitutionSymbol );

	/**
	 * @copydoc FormalRTEElement::clone() const
	 */
	virtual FormalRTESubstitution * clone ( ) const;

	/**
	 * @copydoc FormalRTEElement::plunder() const
	 */
	virtual FormalRTESubstitution * plunder ( ) &&;

	/**
	 * @copydoc FormalRTEElement::testSymbol() const
	 */
	virtual bool testSymbol ( const std::ranked_symbol < > & symbol ) const;

	/**
	 * @copydoc FormalRTEElement::computeMinimalAlphabet()
	 */
	virtual void computeMinimalAlphabet ( std::set < std::ranked_symbol < > > & alphabetF, std::set < std::ranked_symbol < > > & alphabetK ) const;

	/**
	 * @copydoc FormalRTEElement::checkAlphabet()
	 */
	virtual bool checkAlphabet ( const std::set < std::ranked_symbol < > > & alphabetF, const std::set < std::ranked_symbol < > > & alphabetK ) const;

	/**
	 * @return elements
	 */
	const FormalRTEElement & getLeftElement ( ) const;
	const FormalRTEElement & getRightElement ( ) const;
	const FormalRTESymbolSubst & getSubstitutionSymbol ( ) const;

	/**
	 * @return elements
	 */
	FormalRTEElement & getLeftElement ( );
	FormalRTEElement & getRightElement ( );
	FormalRTESymbolSubst & getSubstitutionSymbol ( );

	/**
	 * @param element to append
	 */
	void setLeftElement ( const FormalRTEElement & element );
	void setLeftElement ( FormalRTEElement && element );

	void setRightElement ( const FormalRTEElement & element );
	void setRightElement ( FormalRTEElement && element );

	void setSubstitutionSymbol ( FormalRTESymbolSubst element );

	virtual int compare ( const FormalRTEElement & other ) const {
		if ( std::type_index ( typeid ( * this ) ) == std::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return std::type_index ( typeid ( * this ) ) - std::type_index ( typeid ( other ) );
	}

	virtual int compare ( const FormalRTESubstitution & ) const;

	/**
	 * @copydoc FormalRTEElement::operator>>() const
	 */
	virtual void operator >>( std::ostream & out ) const;
	virtual explicit operator std::string ( ) const;
};

} /* namespace rte */

#endif /* FORMAL_RTE_SUBSTITUTION_H_ */
