#include "FormalRTESymbolAlphabet.h"

#include <primitive/Unsigned.h>
#include <exception/CommonException.h>
#include <sstream>

namespace rte {

FormalRTESymbolAlphabet::FormalRTESymbolAlphabet ( std::ranked_symbol < > symbol ) : FormalRTESymbol ( symbol ) {
}

FormalRTESymbolAlphabet * FormalRTESymbolAlphabet::clone ( ) const {
	return new FormalRTESymbolAlphabet ( * this );
}

FormalRTESymbolAlphabet * FormalRTESymbolAlphabet::plunder ( ) && {
	return new FormalRTESymbolAlphabet ( std::move ( * this ) );
}

const std::vector < std::smart_ptr < const FormalRTESymbol > > & FormalRTESymbolAlphabet::getElements ( ) const {
	return getChildren ( );
}

const std::vector < std::smart_ptr < FormalRTESymbol > > & FormalRTESymbolAlphabet::getElements ( ) {
	return getChildren ( );
}

void FormalRTESymbolAlphabet::appendElement ( const FormalRTESymbol & element ) {
	return appendElement ( std::move_copy ( element ) );
}

void FormalRTESymbolAlphabet::appendElement ( FormalRTESymbol && element ) {
	// TODO: Remove this.  Pass vector in constructor. Now an Element can have LESS children that is MUST have.

	if ( primitive::Unsigned ( getChildren ( ).size ( ) ) >= this->symbol.getRank ( ) )
		throw exception::CommonException ( "Ranked node cannot have more children then its rank" );

	pushBackChild ( std::smart_ptr < FormalRTESymbol > ( std::move ( element ).plunder ( ) ) );
}

int FormalRTESymbolAlphabet::compare ( const FormalRTESymbolAlphabet & other ) const {
	auto first = std::tie ( symbol, getChildren ( ) );
	auto second = std::tie ( other.symbol, other.getChildren ( ) );

	std::compare < decltype ( first ) > comp;

	return comp ( first, second );
}

void FormalRTESymbolAlphabet::operator >>( std::ostream & out ) const {
	out << "(FormalRTESymbolAlphabet " << " symbol = " << this->symbol << " children = " << this->getChildren ( ) << "})";
}

bool FormalRTESymbolAlphabet::testSymbol ( const std::ranked_symbol < > & symbol ) const {
	return symbol == this->symbol;
}

void FormalRTESymbolAlphabet::computeMinimalAlphabet ( std::set < std::ranked_symbol < > > & alphabetF, std::set < std::ranked_symbol < > > & alphabetK ) const {
	alphabetF.insert ( this->symbol );

	for ( const std::smart_ptr < const FormalRTESymbol > & child : getChildren ( ) )
		child->computeMinimalAlphabet ( alphabetF, alphabetK );
}

bool FormalRTESymbolAlphabet::checkAlphabet ( const std::set < std::ranked_symbol < > > & alphabetF, const std::set < std::ranked_symbol < > > & alphabetK ) const {
	return alphabetF.count ( getSymbol ( ) ) > 0 && std::all_of ( getElements ( ).begin ( ), getElements ( ).end ( ), [&] ( const std::smart_ptr < const FormalRTESymbol > & e ) {
			return e->checkAlphabet ( alphabetF, alphabetK );
		} );
}

FormalRTESymbolAlphabet::operator std::string ( ) const {
	std::stringstream ss;
	ss << * this;
	return ss.str ( );
}

} /* namespace rte */
