#ifndef FORMAL_RTE_SYMBOL_ALPHABET_H_
#define FORMAL_RTE_SYMBOL_ALPHABET_H_

#include "FormalRTE.h"
#include "FormalRTESymbol.h"

namespace rte {

/**
 * Represents symbol in the regular expression. Contains name of the symbol.
 */
class FormalRTESymbolAlphabet : public FormalRTESymbol, public std::VararyNode < std::smart_ptr < FormalRTESymbol >, std::smart_ptr < const FormalRTESymbol >, FormalRTESymbolAlphabet > {
public:
	void Accept ( void * userData, const FormalRTEElementVisitor & visitor ) const {
		visitor.Visit ( userData, * this );
	}

	explicit FormalRTESymbolAlphabet ( std::ranked_symbol < > symbol );

	/**
	 * @copydoc FormalRTEElement::clone() const
	 */
	virtual FormalRTESymbolAlphabet * clone ( ) const;

	/**
	 * @copydoc FormalRTEElement::plunder() const
	 */
	virtual FormalRTESymbolAlphabet * plunder ( ) &&;

	/**
	 * @copydoc FormalRTEElement::testSymbol() const
	 */
	virtual bool testSymbol ( const std::ranked_symbol < > & symbol ) const;

	/**
	 * @copydoc FormalRTEElement::computeMinimalAlphabet()
	 */
	virtual void computeMinimalAlphabet ( std::set < std::ranked_symbol < > > & alphabetF, std::set < std::ranked_symbol < > > & alphabetK ) const;

	/**
	 * @copydoc FormalRTEElement::checkAlphabet()
	 */
	virtual bool checkAlphabet ( const std::set < std::ranked_symbol < > > & alphabetF, const std::set < std::ranked_symbol < > > & alphabetK ) const;

	const std::vector < std::smart_ptr < const FormalRTESymbol > > & getElements ( ) const;
	const std::vector < std::smart_ptr < FormalRTESymbol > > & getElements ( );

	void appendElement ( const FormalRTESymbol & element );
	void appendElement ( FormalRTESymbol && element );

	virtual int compare ( const FormalRTESymbolAlphabet & ) const;

	virtual int compare ( const FormalRTEElement & other ) const {
		if ( std::type_index ( typeid ( * this ) ) == std::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return std::type_index ( typeid ( * this ) ) - std::type_index ( typeid ( other ) );
	}

	/**
	 * @copydoc FormalRTEElement::operator>>() const
	 */
	virtual void operator >>( std::ostream & out ) const;

	/**
	 * Returns the string representation of RTE Symbol.
	 */
	virtual explicit operator std::string ( ) const;

	friend class FormalRTEIteration;
	friend class FormalRTESubstitution;
};

} /* namespace rte */

#endif /* FORMAL_RTE_SYMBOL_ALPHABET_H_ */
