/*
 * TreeAuxiliary.cpp
 *
 *  Created on: May 5, 2016
 *      Author: Jan Travnicek
 */

#include "TreeAuxiliary.h"
#include <algorithm>

namespace tree {

std::set < alphabet::Symbol > TreeAuxiliary::unrankSymbols ( const std::set < std::ranked_symbol < > > & alphabet ) {
	return std::transform < alphabet::Symbol > ( alphabet, [&] ( const std::ranked_symbol < > & symbol) {
			return symbol.getSymbol ( );
	} );
}

std::set < std::ranked_symbol < > > TreeAuxiliary::computeBars ( const std::set < std::ranked_symbol < > > & alphabet, const alphabet::Symbol & barBase ) {
	return std::transform < std::ranked_symbol < > > ( alphabet, [&] ( const std::ranked_symbol < > & symbol) {
			return std::ranked_symbol < > ( barBase, symbol.getRank ( ) );
	} );
}

std::tree < std::ranked_symbol < > > TreeAuxiliary::toRankedTree ( const std::tree < alphabet::Symbol > & tree ) {
	std::vector < std::tree < std::ranked_symbol < > > > children;

	for ( const std::tree < alphabet::Symbol > & child : tree ) {
		children.push_back ( toRankedTree ( child ) );
	}

	unsigned size = children.size ( );
	return std::tree < std::ranked_symbol < > > ( std::ranked_symbol < > ( tree.getData ( ), size ), std::move ( children ) );
}

std::tree < alphabet::Symbol > TreeAuxiliary::toUnrankedTree ( const std::tree < std::ranked_symbol < > > & tree ) {
	std::vector < std::tree < alphabet::Symbol > > children;

	for ( const std::tree < std::ranked_symbol < > > & child : tree ) {
		children.push_back ( toUnrankedTree ( child ) );
	}

	return std::tree < alphabet::Symbol > ( tree.getData ( ).getSymbol ( ), std::move ( children ) );
}

} /* namespace tree */
