/*
 * TreeFromXMLParser.cpp
 *
 *  Created on: Nov 16, 2014
 *      Author: Stepan Plachy
 */

#include <sax/FromXMLParserHelper.h>
#include "TreeFromXMLParser.h"
#include <sax/ParserException.h>
#include <core/xmlApi.hpp>
#include <alphabet/RankedSymbol.h>

namespace tree {

alphabet::Symbol TreeFromXMLParser::parseBar ( std::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "bar" );
	alphabet::Symbol bar ( alib::xmlApi < alphabet::Symbol >::parse ( input ) );
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "bar" );

	return bar;
}

std::set < std::ranked_symbol < > > TreeFromXMLParser::parseBars ( std::deque < sax::Token >::iterator & input ) {
	std::set < std::ranked_symbol < > > bars;
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "bars" );

	while ( sax::FromXMLParserHelper::isTokenType ( input, sax::Token::TokenType::START_ELEMENT ) )
		bars.insert ( alib::xmlApi < std::ranked_symbol < > >::parse ( input ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "bars" );

	return bars;
}

alphabet::Symbol TreeFromXMLParser::parseVariablesBar ( std::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "variablesBar" );
	alphabet::Symbol variablesBar ( alib::xmlApi < alphabet::Symbol >::parse ( input ) );
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "variablesBar" );

	return variablesBar;
}

std::ranked_symbol < > TreeFromXMLParser::parseRankedVariablesBar ( std::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "RankedVariablesBar" );
	std::ranked_symbol < > rankedVariablesBar ( alib::xmlApi < std::ranked_symbol < > >::parse ( input ) );
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "RankedVariablesBar" );

	return rankedVariablesBar;
}

alphabet::Symbol TreeFromXMLParser::parseSubtreeWildcardSymbol ( std::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "subtreeWildcard" );
	alphabet::Symbol subtreeWildcard ( alib::xmlApi < alphabet::Symbol >::parse ( input ) );
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "subtreeWildcard" );

	return subtreeWildcard;
}

std::ranked_symbol < > TreeFromXMLParser::parseSubtreeWildcardRawRankedSymbol ( std::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "subtreeWildcard" );
	std::ranked_symbol < > subtreeWildcard ( alib::xmlApi < std::ranked_symbol < > >::parse ( input ) );
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "subtreeWildcard" );

	return subtreeWildcard;
}

std::set < std::ranked_symbol < > > TreeFromXMLParser::parseRankedAlphabet ( std::deque < sax::Token >::iterator & input ) {
	std::set < std::ranked_symbol < > > rankedSymbols;
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "rankedAlphabet" );

	while ( sax::FromXMLParserHelper::isTokenType ( input, sax::Token::TokenType::START_ELEMENT ) )
		rankedSymbols.insert ( alib::xmlApi < std::ranked_symbol < > >::parse ( input ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "rankedAlphabet" );
	return rankedSymbols;
}

std::set < alphabet::Symbol > TreeFromXMLParser::parseUnrankedAlphabet ( std::deque < sax::Token >::iterator & input ) {
	std::set < alphabet::Symbol > symbols;
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "alphabet" );

	while ( sax::FromXMLParserHelper::isTokenType ( input, sax::Token::TokenType::START_ELEMENT ) )
		symbols.insert ( alib::xmlApi < alphabet::Symbol >::parse ( input ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "alphabet" );
	return symbols;
}

std::vector < std::ranked_symbol < > > TreeFromXMLParser::parseRankedLinearContent ( std::deque < sax::Token >::iterator & input ) {
	std::vector < std::ranked_symbol < > > data;
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "content" );

	while ( sax::FromXMLParserHelper::isTokenType ( input, sax::Token::TokenType::START_ELEMENT ) )
		data.push_back ( alib::xmlApi < std::ranked_symbol < > >::parse ( input ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "content" );
	return data;
}

std::vector < alphabet::Symbol > TreeFromXMLParser::parseUnrankedLinearContent ( std::deque < sax::Token >::iterator & input ) {
	std::vector < alphabet::Symbol > data;
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "content" );

	while ( sax::FromXMLParserHelper::isTokenType ( input, sax::Token::TokenType::START_ELEMENT ) )
		data.push_back ( alib::xmlApi < alphabet::Symbol >::parse ( input ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "content" );
	return data;
}

std::tree < std::ranked_symbol < > > TreeFromXMLParser::parseRankedTreeChild ( std::deque < sax::Token >::iterator & input ) {
	std::ranked_symbol < > symbol = alib::xmlApi < std::ranked_symbol < > >::parse ( input );

	std::vector < std::tree < std::ranked_symbol < > > > children;

	if ( sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, "Children" ) ) {
		sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "Children" );

		while ( sax::FromXMLParserHelper::isTokenType ( input, sax::Token::TokenType::START_ELEMENT ) ) {
			children.push_back ( TreeFromXMLParser::parseRankedTreeChild ( input ) );
		}

		sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "Children" );
	}

	return std::tree < std::ranked_symbol < > > ( symbol, std::move( children ) );
}

std::tree < std::ranked_symbol < > > TreeFromXMLParser::parseRankedTreeContent ( std::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "content" );

	std::tree < std::ranked_symbol < > > data = TreeFromXMLParser::parseRankedTreeChild ( input );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "content" );
	return data;
}

std::tree < alphabet::Symbol > TreeFromXMLParser::parseUnrankedTreeContent ( std::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "content" );

	std::tree < alphabet::Symbol > data ( alib::xmlApi < alphabet::Symbol >::parse ( input ) );

	std::tree < alphabet::Symbol >::const_children_iterator under = data.root ( );

	unsigned level = 0;

	while ( sax::FromXMLParserHelper::isTokenType ( input, sax::Token::TokenType::START_ELEMENT ) ) {
		if ( sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, "Children" ) ) {
			sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "Children" );

			if ( level > 0 )
				under = std::prev ( under->end ( ) );

			++level;
		}

		data.push_back ( under, alib::xmlApi < alphabet::Symbol >::parse ( input ) );

		while ( sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::END_ELEMENT, "Children" ) ) {
			sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "Children" );

			under = under->parent ( );
			--level;
		}
	}

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "content" );
	return data;
}

std::set < std::ranked_symbol < > > TreeFromXMLParser::parseRankedNonlinearVariables ( std::deque < sax::Token >::iterator & input ) {
	std::set < std::ranked_symbol < > > rankedSymbols;
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "rankedNonlinearVariables" );

	while ( sax::FromXMLParserHelper::isTokenType ( input, sax::Token::TokenType::START_ELEMENT ) )
		rankedSymbols.insert ( alib::xmlApi < std::ranked_symbol < > >::parse ( input ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "rankedNonlinearVariables" );
	return rankedSymbols;
}

std::set < alphabet::Symbol > TreeFromXMLParser::parseUnrankedNonlinearVariables ( std::deque < sax::Token >::iterator & input ) {
	std::set < alphabet::Symbol > symbols;
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "nonlinearVariables" );

	while ( sax::FromXMLParserHelper::isTokenType ( input, sax::Token::TokenType::START_ELEMENT ) )
		symbols.insert ( alib::xmlApi < alphabet::Symbol >::parse ( input ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "nonlinearVariables" );
	return symbols;
}

} /* namespace tree */
