/*
 * TreeFromXMLParser.h
 *
 *  Created on: Nov 16, 2014
 *      Author: Stepan Plachy
 */

#ifndef TREE_FROM_XML_PARSER_H_
#define TREE_FROM_XML_PARSER_H_

#include <set>
#include <vector>
#include <deque>
#include <tree>

#include <sax/Token.h>
#include "../../alphabet/SymbolFeatures.h"
#include <alphabet/ranked_symbol.hpp>

namespace tree {

/**
 * Parser used to get tree from XML parsed into list of Tokens.
 */
class TreeFromXMLParser {
	static std::tree < std::ranked_symbol < > > parseRankedTreeChild ( std::deque < sax::Token >::iterator & input );

public:
	static std::vector < std::ranked_symbol < > > parseRankedLinearContent ( std::deque < sax::Token >::iterator & input );
	static std::vector < alphabet::Symbol > parseUnrankedLinearContent ( std::deque < sax::Token >::iterator & input );
	static std::tree < alphabet::Symbol > parseUnrankedTreeContent ( std::deque < sax::Token >::iterator & input );
	static std::tree < std::ranked_symbol < > > parseRankedTreeContent ( std::deque < sax::Token >::iterator & input );

	static std::set < std::ranked_symbol < > > parseRankedAlphabet ( std::deque < sax::Token >::iterator & input );
	static std::set < alphabet::Symbol > parseUnrankedAlphabet ( std::deque < sax::Token >::iterator & input );

	static alphabet::Symbol parseBar ( std::deque < sax::Token >::iterator & input );
	static std::set < std::ranked_symbol < > > parseBars ( std::deque < sax::Token >::iterator & input );
	static alphabet::Symbol parseVariablesBar ( std::deque < sax::Token >::iterator & input );
	static std::ranked_symbol < > parseRankedVariablesBar ( std::deque < sax::Token >::iterator & input );

	static alphabet::Symbol parseSubtreeWildcardSymbol ( std::deque < sax::Token >::iterator & input );
	static std::ranked_symbol < > parseSubtreeWildcardRawRankedSymbol ( std::deque < sax::Token >::iterator & input );

	static std::set < std::ranked_symbol < > > parseRankedNonlinearVariables ( std::deque < sax::Token >::iterator & input );
	static std::set < alphabet::Symbol > parseUnrankedNonlinearVariables ( std::deque < sax::Token >::iterator & input );
};

} /* namespace tree */

#endif /* TREE_FROM_XML_PARSER_H_ */
