/*
 * TreeToXMLComposer.cpp
 *
 *  Created on: Nov 16, 2014
 *      Author: Stepan Plachy
 */

#include "TreeToXMLComposer.h"
#include <core/xmlApi.hpp>
#include <alphabet/RankedSymbol.h>

namespace tree {

void TreeToXMLComposer::composeBar ( std::deque < sax::Token > & out, const alphabet::Symbol & symbol ) {
	out.emplace_back ( sax::Token ( "bar", sax::Token::TokenType::START_ELEMENT ) );
	alib::xmlApi < alphabet::Symbol >::compose ( out, symbol );
	out.emplace_back ( sax::Token ( "bar", sax::Token::TokenType::END_ELEMENT ) );
}

void TreeToXMLComposer::composeBars ( std::deque < sax::Token > & out, const std::set < std::ranked_symbol < > > & symbols ) {
	out.emplace_back ( sax::Token ( "bars", sax::Token::TokenType::START_ELEMENT ) );

	for ( const auto & symbol : symbols )
		alib::xmlApi < std::ranked_symbol < > >::compose ( out, symbol );

	out.emplace_back ( sax::Token ( "bars", sax::Token::TokenType::END_ELEMENT ) );
}

void TreeToXMLComposer::composeVariablesBar ( std::deque < sax::Token > & out, const alphabet::Symbol & symbol ) {
	out.emplace_back ( sax::Token ( "variablesBar", sax::Token::TokenType::START_ELEMENT ) );
	alib::xmlApi < alphabet::Symbol >::compose ( out, symbol );
	out.emplace_back ( sax::Token ( "variablesBar", sax::Token::TokenType::END_ELEMENT ) );
}

void TreeToXMLComposer::composeVariablesBar ( std::deque < sax::Token > & out, const std::ranked_symbol < > & symbol ) {
	out.emplace_back ( sax::Token ( "RankedVariablesBar", sax::Token::TokenType::START_ELEMENT ) );
	alib::xmlApi < std::ranked_symbol < > >::compose ( out, symbol );
	out.emplace_back ( sax::Token ( "RankedVariablesBar", sax::Token::TokenType::END_ELEMENT ) );
}

void TreeToXMLComposer::composeSubtreeWildcard ( std::deque < sax::Token > & out, const alphabet::Symbol & symbol ) {
	out.emplace_back ( sax::Token ( "subtreeWildcard", sax::Token::TokenType::START_ELEMENT ) );
	alib::xmlApi < alphabet::Symbol >::compose ( out, symbol );
	out.emplace_back ( sax::Token ( "subtreeWildcard", sax::Token::TokenType::END_ELEMENT ) );
}

void TreeToXMLComposer::composeSubtreeWildcard ( std::deque < sax::Token > & out, const std::ranked_symbol < > & symbol ) {
	out.emplace_back ( sax::Token ( "subtreeWildcard", sax::Token::TokenType::START_ELEMENT ) );
	alib::xmlApi < std::ranked_symbol < > >::compose ( out, symbol );
	out.emplace_back ( sax::Token ( "subtreeWildcard", sax::Token::TokenType::END_ELEMENT ) );
}

void TreeToXMLComposer::composeAlphabet ( std::deque < sax::Token > & out, const std::set < std::ranked_symbol < > > & symbols ) {
	out.emplace_back ( sax::Token ( "rankedAlphabet", sax::Token::TokenType::START_ELEMENT ) );

	for ( const auto & symbol : symbols )
		alib::xmlApi < std::ranked_symbol < > >::compose ( out, symbol );

	out.emplace_back ( sax::Token ( "rankedAlphabet", sax::Token::TokenType::END_ELEMENT ) );
}

void TreeToXMLComposer::composeAlphabet ( std::deque < sax::Token > & out, const std::set < alphabet::Symbol > & symbols ) {
	out.emplace_back ( sax::Token ( "alphabet", sax::Token::TokenType::START_ELEMENT ) );

	for ( const auto & symbol : symbols )
		alib::xmlApi < alphabet::Symbol >::compose ( out, symbol );

	out.emplace_back ( sax::Token ( "alphabet", sax::Token::TokenType::END_ELEMENT ) );
}

void TreeToXMLComposer::composeContent ( std::deque < sax::Token > & out, const std::vector < alphabet::Symbol > & content ) {
	out.emplace_back ( "content", sax::Token::TokenType::START_ELEMENT );

	for ( const auto & symbol : content )
		alib::xmlApi < alphabet::Symbol >::compose ( out, symbol );

	out.emplace_back ( "content", sax::Token::TokenType::END_ELEMENT );
}

void TreeToXMLComposer::composeContent ( std::deque < sax::Token > & out, const std::tree < alphabet::Symbol > & content ) {
	out.emplace_back ( "content", sax::Token::TokenType::START_ELEMENT );

	unsigned level = 0;

	for ( std::tree < alphabet::Symbol >::const_prefix_iterator iter = content.prefix_begin ( ); iter != content.prefix_end ( ); ) {
		while ( iter.getLevel ( ) > level ) {
			out.emplace_back ( "Children", sax::Token::TokenType::START_ELEMENT );
			++level;
		}

		alib::xmlApi < alphabet::Symbol >::compose ( out, * iter );
		++iter;

		while ( iter.getLevel ( ) < level ) {
			out.emplace_back ( "Children", sax::Token::TokenType::END_ELEMENT );
			--level;
		}
	}

	out.emplace_back ( "content", sax::Token::TokenType::END_ELEMENT );
}

void TreeToXMLComposer::composeContent ( std::deque < sax::Token > & out, const std::tree < std::ranked_symbol < > > & content ) {
	out.emplace_back ( "content", sax::Token::TokenType::START_ELEMENT );

	unsigned level = 0;

	for ( std::tree < std::ranked_symbol < > >::const_prefix_iterator iter = content.prefix_begin ( ); iter != content.prefix_end ( ); ) {
		while ( iter.getLevel ( ) > level ) {
			out.emplace_back ( "Children", sax::Token::TokenType::START_ELEMENT );
			++level;
		}

		alib::xmlApi < std::ranked_symbol < > >::compose ( out, * iter );
		++iter;

		while ( iter.getLevel ( ) < level ) {
			out.emplace_back ( "Children", sax::Token::TokenType::END_ELEMENT );
			--level;
		}
	}

	out.emplace_back ( "content", sax::Token::TokenType::END_ELEMENT );
}

void TreeToXMLComposer::composeContent ( std::deque < sax::Token > & out, const std::vector < std::ranked_symbol < > > & content ) {
	out.emplace_back ( "content", sax::Token::TokenType::START_ELEMENT );

	for ( const auto & symbol : content )
		alib::xmlApi < std::ranked_symbol < > >::compose ( out, symbol );

	out.emplace_back ( "content", sax::Token::TokenType::END_ELEMENT );
}

void TreeToXMLComposer::composeNonlinearVariables ( std::deque < sax::Token > & out, const std::set < std::ranked_symbol < > > & symbols ) {
	out.emplace_back ( sax::Token ( "rankedNonlinearVariables", sax::Token::TokenType::START_ELEMENT ) );

	for ( const auto & symbol : symbols )
		alib::xmlApi < std::ranked_symbol < > >::compose ( out, symbol );

	out.emplace_back ( sax::Token ( "rankedNonlinearVariables", sax::Token::TokenType::END_ELEMENT ) );
}

void TreeToXMLComposer::composeNonlinearVariables ( std::deque < sax::Token > & out, const std::set < alphabet::Symbol > & symbols ) {
	out.emplace_back ( sax::Token ( "nonlinearVariables", sax::Token::TokenType::START_ELEMENT ) );

	for ( const auto & symbol : symbols )
		alib::xmlApi < alphabet::Symbol >::compose ( out, symbol );

	out.emplace_back ( sax::Token ( "nonlinearVariables", sax::Token::TokenType::END_ELEMENT ) );
}

} /* namespace tree */
