/*
 * TreeToXMLComposer.h
 *
 *  Created on: Nov 16, 2014
 *      Author: Stepan Plachy
 */

#ifndef TREE_TO_XML_COMPOSER_H_
#define TREE_TO_XML_COMPOSER_H_

#include <deque>
#include <set>
#include <vector>
#include <tree>
#include "../../alphabet/SymbolFeatures.h"
#include <sax/Token.h>
#include <alphabet/ranked_symbol.hpp>

namespace tree {

/**
 * This class contains methods to print XML representation of tree to the output stream.
 */
class TreeToXMLComposer {
public:
	static void composeAlphabet ( std::deque < sax::Token > & out, const std::set < std::ranked_symbol < > > & symbols );
	static void composeAlphabet ( std::deque < sax::Token > & out, const std::set < alphabet::Symbol > & symbols );

	static void composeBar ( std::deque < sax::Token > & out, const alphabet::Symbol & symbol );
	static void composeBars ( std::deque < sax::Token > & out, const std::set < std::ranked_symbol < > > & symbols );
	static void composeVariablesBar ( std::deque < sax::Token > & out, const alphabet::Symbol & symbol );
	static void composeVariablesBar ( std::deque < sax::Token > & out, const std::ranked_symbol < > & symbol );

	static void composeSubtreeWildcard ( std::deque < sax::Token > & out, const alphabet::Symbol & symbol );
	static void composeSubtreeWildcard ( std::deque < sax::Token > & out, const std::ranked_symbol < > & symbol );

	static void composeContent ( std::deque < sax::Token > & out, const std::vector < alphabet::Symbol > & content );
	static void composeContent ( std::deque < sax::Token > & out, const std::tree < alphabet::Symbol > & content );
	static void composeContent ( std::deque < sax::Token > & out, const std::vector < std::ranked_symbol < > > & content );
	static void composeContent ( std::deque < sax::Token > & out, const std::tree < std::ranked_symbol < > > & content );

	static void composeNonlinearVariables ( std::deque < sax::Token > & out, const std::set < std::ranked_symbol < > > & symbols );
	static void composeNonlinearVariables ( std::deque < sax::Token > & out, const std::set < alphabet::Symbol > & symbols );
};

} /* namespace tree */

#endif /* TREE_TO_XML_COMPOSER_H_ */
