/*
 * AutomatonFromStringParser.h
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#ifndef AUTOMATON_FROM_STRING_PARSER_H_
#define AUTOMATON_FROM_STRING_PARSER_H_

#include "AutomatonFromStringLexer.h"

#include "automaton/AutomatonFeatures.h"
#include "string/StringFeatures.h"
#include "alphabet/SymbolFeatures.h"
#include "label/LabelFeatures.h"

#include <set>
#include <vector>
#include <tuple>
#include <variant>

namespace automaton {

class AutomatonFromStringParser {
	static void initialFinalState(std::istream& input, bool& initial, bool& final);

	static void parseEpsilonNFATransition(std::istream& input, std::set<label::Label>& states, const std::vector<std::variant<string::Epsilon < >, alphabet::Symbol>>& symbols, label::Label*& initialState, std::set<label::Label>& finalStates, std::set<std::tuple<label::Label, std::variant<string::Epsilon < >, alphabet::Symbol>, label::Label>>& transitionFunction);
	static void parseMultiInitialStateNFATransition(std::istream& input, MultiInitialStateNFA < > & res, const std::vector<alphabet::Symbol>& symbols);
	static void parseNFATransition(std::istream& input, std::set<label::Label>& states, const std::vector<alphabet::Symbol>& symbols, label::Label*& initialState, std::set<label::Label>& finalStates, std::set<std::tuple<label::Label, alphabet::Symbol, label::Label>>& transitionFunction);
	static void parseDFATransition(std::istream& input, std::set<label::Label>& states, const std::vector<alphabet::Symbol>& symbols, label::Label*& initialState, std::set<label::Label>& finalStates, std::set<std::tuple<label::Label, alphabet::Symbol, label::Label>>& transitionFunction);

public:
	static Automaton parseAutomaton(std::istream& input);
	static Automaton parseAutomaton(std::istream& input, const std::set<FEATURES>& features);

	static EpsilonNFA<> parseEpsilonNFA(std::istream& input);
	static NFA<> parseNFA(std::istream& input);
	static MultiInitialStateNFA<> parseMultiInitialStateNFA(std::istream& input);
	static DFA<> parseDFA(std::istream& input);
};

} /* namespace automaton */

#endif /* AUTOMATON_FROM_STRING_PARSER_H_ */
