/*
 * AutomatonToStringComposer.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#ifndef AUTOMATON_TO_STRING_COMPOSER_H_
#define AUTOMATON_TO_STRING_COMPOSER_H_

#include <ostream>
#include <core/multipleDispatch.hpp>
#include "automaton/Automaton.h"
#include "automaton/AutomatonFeatures.h"
#include "label/LabelFeatures.h"

namespace automaton {

class AutomatonToStringComposer : public std::SingleDispatchFirstStaticParam<AutomatonToStringComposer, void, std::ostream&, const AutomatonBase &> {
	static void composeTransitionsFromState(std::ostream& out, const DFA < > & automaton, const label::Label& from);
	static void composeTransitionsFromState(std::ostream& out, const NFA < > & automaton, const label::Label& from);
	static void composeTransitionsFromState(std::ostream& out, const MultiInitialStateNFA < > & automaton, const label::Label& from);
	static void composeTransitionsFromState(std::ostream& out, const EpsilonNFA < > & automaton, const label::Label& from);

public:
	/**
	 * Prints XML representation of UnknownAutomaton to the output stream.
	 * @param automaton automaton to print
	 * @return list of xml tokens representing the automaton
	 */
	static void compose(std::ostream& output, const Automaton& automaton);

	static void compose(std::ostream& output, const DFA < > & automaton);
	static void compose(std::ostream& output, const NFA < > & automaton);
	static void compose(std::ostream& output, const MultiInitialStateNFA < > & automaton);
	static void compose(std::ostream& output, const EpsilonNFA < > & automaton);
};

} /* namespace automaton */

#endif /* AUTOMATON_TO_STRING_COMPOSER_H_ */
